/*  ClockViewer for Win32  */
/*  copyright (c) 2005 Bee! (mailto:bee@mbj.nifty.com)  */

#include "base.h"

static HDC vhdcBack;
static HBITMAP vhbmpBack;
static HBITMAP vhbmpOld;

static void OnPaint( HWND hwnd, HDC dc );
static void OnDestroy( HWND hwnd );
static void OnMouseMove(HWND hwnd, WPARAM wParam, LPARAM lParam );
static void OnKeyDown(HWND hwnd, WPARAM wParam, LPARAM lParam );
static void OnLButtonDown(HWND hwnd );
static void ApplyClose( HWND hwnd );

static HWND vhwndAdjust;

static int cur_mode;
static int moved;

SIZE vPanel;

int DoAdjustCPU( HWND hwndParent, POINT *pt ){
	HINSTANCE hInstance;
	HWND hwnd;
	SIZE sz;
	HDC dc;
	unsigned int n;
	SYSTEM_POWER_STATUS sps;
	MACHINE_PROCESSOR_POWER_POLICY mp;

	if( !vfAdjustable ){
		return 0;
	}

	if( vhwndAdjust ){
		SetFocus( vhwndAdjust );
		return 0;
	}

	GetActivePwrScheme( &n );
	(*pfnReadProcessorPwrScheme)( n, &mp );

	GetSystemPowerStatus( &sps );
	if( sps.ACLineStatus == 0 ){
		n = mp.ProcessorPolicyDc.DynamicThrottle;
	}else{
		n = mp.ProcessorPolicyAc.DynamicThrottle;
	}
	switch( n ){
	case PO_THROTTLE_NONE:
		cur_mode = 0;
		break;
	case PO_THROTTLE_ADAPTIVE:
		cur_mode = 1;
		break;
	case PO_THROTTLE_CONSTANT:
		cur_mode = 2;
		break;
	case PO_THROTTLE_DEGRADE:
		cur_mode = 3;
		break;
	default:
		cur_mode = 0;
	}
	moved = 0;

	hInstance = GetModuleHandle( NULL );

	vhbmpBack = BMPLoad( "./select.bmp", &sz );
	if( !vhbmpBack ){
		return FALSE;
	}
	vPanel.cx = sz.cx / 2;
	vPanel.cy = sz.cy;

	hwnd = CreateWindowEx(WS_EX_TOPMOST,
						  WINDOW_CLASS, "BClock", WS_POPUP,
						  pt->x, pt->y, vPanel.cx, sz.cy, hwndParent, NULL, hInstance, NULL);


	if( hwnd == NULL ){
		return 0;
	}
	SetWindowLong( hwnd, GWL_WNDPROC, (long int)AdjustWndProc );
	dc = GetDC( hwnd );
	vhdcBack = CreateCompatibleDC( dc );
	ReleaseDC( dc, hwnd );

	vhbmpOld = SelectObject( vhdcBack, vhbmpBack );

	ShowWindow( hwnd, SW_SHOW );
	UpdateWindow( hwnd );

	vhwndAdjust = hwnd;

	return 0;
}


LRESULT CALLBACK AdjustWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam){
	PAINTSTRUCT ps;
	HDC dc;

	switch(uMsg){
	case WM_PAINT:
		dc = BeginPaint(hwnd, &ps);
		OnPaint(hwnd, dc);
		EndPaint(hwnd, &ps);
		break;

	case WM_DESTROY:
		OnDestroy( hwnd );
		break;

	case WM_KILLFOCUS:
		DestroyWindow( hwnd );
		break;

	case WM_MOUSEMOVE:
		OnMouseMove( hwnd, wParam, lParam );
		break;

	case WM_LBUTTONDOWN:
		OnLButtonDown( hwnd );
		break;

	case WM_KEYDOWN:
		OnKeyDown( hwnd, wParam, lParam );
		break;


	default:
		return DefWindowProc(hwnd, uMsg, wParam, lParam);
	}
	return 0;
}


void OnDestroy( HWND hwnd ){
	vhwndAdjust = NULL;
	SelectObject( vhdcBack, vhbmpOld );
	DeleteObject( vhbmpBack );
	DeleteDC( vhdcBack );
}


void OnPaint(HWND hwnd, HDC dc){
	int y;
	int cy;

	BitBlt( dc, 0, 0, vPanel.cx, vPanel.cy, vhdcBack, 0, 0, SRCCOPY );
	cy = vPanel.cy / 4;
	y = cur_mode * cy;

	BitBlt( dc, 0, y, vPanel.cx, cy, vhdcBack, vPanel.cx, y, SRCCOPY );

	return;
}

void OnMouseMove(HWND hwnd, WPARAM wParam, LPARAM lParam ){
	int n;

	if( !moved ){
		moved = 1;
		return;
	}

	n = HIWORD( lParam ) / ( vPanel.cy / 4 );
	if( n != cur_mode ){
		cur_mode = n;
		InvalidateRect( hwnd, NULL, FALSE );
	}
}

void OnKeyDown(HWND hwnd, WPARAM wParam, LPARAM lParam ){
	int n;

	if( (lParam >> 30) & 0x01 ){
		return;
	}
	n = cur_mode;

	switch( wParam ){
	case 'J':
	case VK_DOWN:
		n++;
		break;
	case 'K':
	case VK_UP:
		n--;
		break;
	case VK_RETURN:
		ApplyClose( hwnd );
		/*THRU*/
	case VK_ESCAPE:
		DestroyWindow( hwnd );
		break;
	}
	if( 0 <= n && n < 4 ){
		cur_mode = n;
		InvalidateRect( hwnd, NULL, FALSE );
	}
}

void OnLButtonDown(HWND hwnd ){
	ApplyClose( hwnd );
	DestroyWindow( hwnd );
}

void ApplyClose( HWND hwnd ){
	HWND parent;

	parent = GetWindow( hwnd, GW_OWNER );
	PostMessage( parent, WM_COMMAND, IDM_CPUSTATE, cur_mode );
}

