/*  ClockViewer for Win32  */
/*  copyright (c) 2005 Bee! (mailto:bee@mbj.nifty.com)  */
/*WriteProcessorPwrScheme*/
#include "base.h"

#include <mmsystem.h>

#define INSPECT_INTERVAL 300

NOTIFYICONDATA ni;

int vnCharSizeX = 12;
int vnCharSizeY = 20;
int vfAdjustable;
static HANDLE vevSync;

struct LAPTIMES vLaps;
BOOL WINAPI (*pfnReadProcessorPwrScheme)(UINT,PMACHINE_PROCESSOR_POWER_POLICY);
BOOL WINAPI (*pfnWriteProcessorPwrScheme)(UINT,PMACHINE_PROCESSOR_POWER_POLICY);

static int vLoop;
static HANDLE vhThread;
static ATOM MyRegisterClass(HINSTANCE hInst);
static BOOL InitInstance(HINSTANCE hInstance, int nCmdShow);
static BOOL WINAPI DummyRWPwrScheme(UINT n, PMACHINE_PROCESSOR_POWER_POLICY ptr);

static LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);


void StartInspeceClock( void );
void StopInspeceClock( void );
static DWORD WINAPI InspectClock( void *param );


void Entry(void){
	MSG msg;
	HINSTANCE hInst;

	hInst = GetModuleHandle( NULL );

	MyRegisterClass( hInst );

	if(!InitInstance( hInst, SW_SHOW )){
		ExitProcess(0);
	}

	for( ;; ){
		switch( MsgWaitForMultipleObjects( 1, &vevSync, FALSE, INFINITE, QS_ALLINPUT ) ){
		case WAIT_OBJECT_0:
			UpdateClock();
			break;
		default:
			while( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE )){
				if( msg.message == WM_QUIT ){
					goto EXIT;
				}
				TranslateMessage( &msg );
				DispatchMessage( &msg );
			}
		}
	}
EXIT:
	StopInspeceClock();
	CloseHandle( vevSync );
	Shell_NotifyIcon( NIM_DELETE, &ni );
	ExitProcess(0);
}


#ifdef _DEBUG

int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR pszCmdLine, int nShow){
	Entry();
	return 0;
}

#endif

ATOM MyRegisterClass(HINSTANCE hInst){
	WNDCLASS wc;

	wc.style		= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc	= (WNDPROC)WndProc;
	wc.cbClsExtra	= 0;
	wc.cbWndExtra	= 0;
	wc.hInstance	= hInst;
	wc.hIcon		= LoadIcon(NULL, IDI_APPLICATION);
	wc.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground= (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName	= NULL;
	wc.lpszClassName= WINDOW_CLASS;

	if( !RegisterClass( &wc )){
		return 0;
	}

	return 1;
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow){
	HANDLE hdllPwrProf;
	HWND hwnd;

	hdllPwrProf = GetModuleHandle( "powrprof.dll" );
	vfAdjustable = 1;
	if( hdllPwrProf ){
		pfnReadProcessorPwrScheme = GetProcAddress( hdllPwrProf, "ReadProcessorPwrScheme" );
		pfnWriteProcessorPwrScheme = GetProcAddress( hdllPwrProf, "WriteProcessorPwrScheme" );
	}
	if( !pfnReadProcessorPwrScheme || !pfnWriteProcessorPwrScheme ){
		vfAdjustable = 0;
		pfnReadProcessorPwrScheme = DummyRWPwrScheme;
		pfnWriteProcessorPwrScheme = DummyRWPwrScheme;
	}

	hwnd = CreateWindowEx(
		0,
		WINDOW_CLASS, "BClock", WS_POPUP,
		CW_USEDEFAULT, 0, 0, 0, NULL, NULL, hInstance, NULL);
	if( !hwnd ){
		return FALSE;
	}

	SetWindowLong( hwnd, GWL_WNDPROC, (long int)baseWndProc );

	ni.cbSize = sizeof( ni );
	ni.hWnd = hwnd;
	ni.uID = 1;
	ni.uFlags = NIF_MESSAGE | NIF_TIP;
	ni.uCallbackMessage = NI_CLICK;

	ni.hIcon = LoadIcon( NULL, IDI_APPLICATION );
	lstrcpy( ni.szTip, vfAdjustable ? "CPU:Adjustable" : "CPU:Not adjustable" );
	Shell_NotifyIcon( NIM_ADD, &ni );
	vevSync = CreateEvent( NULL, FALSE, FALSE, NULL );
	ClockWindow( hwnd );

	return TRUE;
}

BOOL WINAPI DummyRWPwrScheme(UINT n, PMACHINE_PROCESSOR_POWER_POLICY ptr){
	return FALSE;
}

void StartInspeceClock( void ){
	DWORD tid;

	if( vhThread ){
		return;
	}
	vLoop = 1;
	vhThread = CreateThread( NULL, 0, InspectClock, NULL, 0, &tid );
}

void StopInspeceClock( void ){
	if( vhThread ){
		vLoop = 0;
		WaitForSingleObject( vhThread, INFINITE );
		CloseHandle( vhThread );
		vhThread = NULL;
	}
}

int start;
int stop;
static DWORD WINAPI InspectClock( void *param ){
	int i, j;

	start++;
	InitializeCriticalSection( &vLaps.lock );
	vLaps.start = 0;

	timeBeginPeriod( 1 );

	EnterCriticalSection( &vLaps.lock );
	for( i = 0; i < LAP_MAX; ++i ){
		rdtsc( &vLaps.item[ i ].clk );
		vLaps.item[ i ].tm = timeGetTime();
	}
	LeaveCriticalSection( &vLaps.lock );

	vLaps.start = 1;
	vLaps.cur = 0;

	i = 0;
	j = 1;
	while( vLoop ){
		EnterCriticalSection( &vLaps.lock );
		rdtsc( &vLaps.item[ i ].clk );
		vLaps.item[ i ].tm = timeGetTime();
		vLaps.cur = i;
		vLaps.start = j;
		LeaveCriticalSection( &vLaps.lock );

		SetEvent( vevSync );

		i = j;
		if( LAP_MAX <= ++j ){
			j = 0;
		}

		Sleep( INSPECT_INTERVAL );
	}

	timeEndPeriod( 1 );
	DeleteCriticalSection( &vLaps.lock );
	stop++;

	return 0;
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam){
	switch(uMsg){
	case WM_CREATE:
		break;

	case WM_PAINT:
		break;

	case WM_SIZE:
		break;

	case WM_DESTROY:
		PostQuitMessage( 0 );
		break;

	case WM_KEYDOWN:
		break;

	case WM_NCRBUTTONDOWN:
		break;

	case WM_COMMAND:
		break;
	default:
		return DefWindowProc(hwnd, uMsg, wParam, lParam);
	}
	return 0;
}

void SetCpuState( int stat ){
	unsigned int n;
	int t;
	MACHINE_PROCESSOR_POWER_POLICY mp;
	SYSTEM_POWER_STATUS sps;

	if( !vfAdjustable ){
		return;
	}

	switch( stat ){
	case 0:
		t = PO_THROTTLE_NONE;
		break;
	case 1:
		t = PO_THROTTLE_ADAPTIVE;
		break;
	case 2:
		t = PO_THROTTLE_CONSTANT;
		break;
	case 3:
		t = PO_THROTTLE_DEGRADE;
		break;
	default:
		return;
	}

	GetActivePwrScheme( &n );
	(*pfnReadProcessorPwrScheme)( n, &mp );
	GetSystemPowerStatus( &sps );

	if( sps.ACLineStatus == 0 ){
		mp.ProcessorPolicyDc.DynamicThrottle = t;
	}else{
		mp.ProcessorPolicyAc.DynamicThrottle = t;
	}
	(*pfnWriteProcessorPwrScheme)( n, &mp );
	SetActivePwrScheme( n, NULL, NULL );
}

