/*  ClockViewer for Win32  */
/*  copyright (c) 2005 Bee! (mailto:bee@mbj.nifty.com)  */

#include "base.h"
enum{
	IDM_QUIT = 2048,
	IDM_SHOW,
	IDM_CPUNONE,
	IDM_CPUADAPTIVE,
	IDM_CPUCONSTANT,
	IDM_CPUDEGRADE,
};
struct menu_tag{
	int use;
	int id;
	char *str;
};

#define MF_ADJUSTABLE 2
#define MF_SHOW       4
#define MF_HIDE       8

static struct menu_tag menu_table[] ={
	{ 1, IDM_QUIT,			"Quit" },
	{ MF_HIDE, IDM_SHOW,			"Show Clock" },
	{ MF_SHOW, IDM_SHOW,			"Hide Clock" },
	{ MF_ADJUSTABLE, IDM_CPUNONE,		"None"},
	{ MF_ADJUSTABLE, IDM_CPUADAPTIVE,	"Adaptive"},
	{ MF_ADJUSTABLE, IDM_CPUCONSTANT,	"Constant"},
	{ MF_ADJUSTABLE, IDM_CPUDEGRADE,	"Degrade"},
};
HMENU CreateTraysMenu( void );

void OnIconClick( HWND hwnd, WPARAM wParam, LPARAM lParam );
void OnCommand( HWND hwnd, WPARAM wParam );


LRESULT CALLBACK baseWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam){
	switch(uMsg){
	case NI_CLICK:
		OnIconClick( hwnd, wParam, lParam );
		break;
	case WM_PAINT:
		break;

	case WM_SIZE:
		break;

	case WM_DESTROY:
		PostQuitMessage( 0 );
		break;

	case WM_KEYDOWN:
		break;

	case WM_NCRBUTTONDOWN:
		break;

	case WM_COMMAND:
		OnCommand( hwnd, wParam );
		break;

	default:
		return DefWindowProc(hwnd, uMsg, wParam, lParam);
	}
	return 0;
}

void OnIconClick( HWND hwnd, WPARAM wParam, LPARAM lParam ){
	HMENU hMenu;
	POINT pt;

	if( wParam != 1 ){
		return;
	}

	switch( lParam ){
	case WM_LBUTTONDOWN:
		ClockWindow( hwnd );
		break;
	case WM_RBUTTONDOWN:
		hMenu = CreateTraysMenu();
		SetForegroundWindow( hwnd );
		GetCursorPos( &pt );
		TrackPopupMenu( hMenu, 0, pt.x, pt.y, 0, hwnd, NULL );
		DestroyMenu( hMenu );
		break;
	}

}

void OnCommand( HWND hwnd, WPARAM wParam ){
	switch( wParam ){
	case IDM_QUIT:
		DestroyWindow( hwnd );
		break;
	case IDM_SHOW:
		ClockWindow( hwnd );
		break;
	case IDM_CPUNONE:
		SetCpuState( 0 );
		break;
	case IDM_CPUADAPTIVE:
		SetCpuState( 1 );
		break;
	case IDM_CPUCONSTANT:
		SetCpuState( 2 );
		break;
	case IDM_CPUDEGRADE:
		SetCpuState( 3 );
		break;
	}
}

HMENU CreateTraysMenu( void ){
	int n;
	HMENU hMenu;
	int f;

	f = 1;
	if( vfAdjustable ){
		f |= MF_ADJUSTABLE;
	}
	if( vhwndClock ){
		f |= MF_SHOW;
	}else{
		f |= MF_HIDE;
	}

	hMenu = CreatePopupMenu();
	for( n = 0; n < sizeof( menu_table ) / sizeof( menu_table[0]); n++ ){
		if( menu_table[n].use & f ){
			AppendMenu( hMenu, MF_STRING, menu_table[n].id , menu_table[n].str );
		}
	}

	return hMenu;
}



