/*  ClockViewer for Win32  */
/*  copyright (c) 2005 Bee! (mailto:bee@mbj.nifty.com)  */
/*WriteProcessorPwrScheme*/

#include "base.h"

#define BM_SIG  (0x4d42)
#define WIDTHBYTES(i)   ((i+31)/32*4)

static BITMAPINFO *LoadBMP(HANDLE hFile);

static WORD UsedColor(WORD bit){
	switch(bit){
	case 1:
		return 2;
	case 4:
		return 16;
	case 8:
		return 256;
	default:
		return 0;
	}
}

static void CoreToInfo(BITMAPINFOHEADER *pbi){
	DWORD dwWidth;
	DWORD dwHeight;
	WORD wPlanes;
	WORD wBitCount;
	WORD cl;

	dwWidth=((BITMAPCOREHEADER*)pbi)->bcWidth;
	dwHeight=((BITMAPCOREHEADER*)pbi)->bcHeight;
	wPlanes=((BITMAPCOREHEADER*)pbi)->bcPlanes;
	wBitCount=((BITMAPCOREHEADER*)pbi)->bcBitCount;

	pbi->biSize=sizeof(BITMAPINFOHEADER);
	pbi->biWidth=dwWidth;
	pbi->biHeight=dwHeight;
	pbi->biPlanes=wPlanes;
	pbi->biBitCount=wBitCount;

	cl=UsedColor(wBitCount);
	pbi->biCompression=BI_RGB;
	pbi->biSizeImage=0;
	pbi->biXPelsPerMeter=0;
	pbi->biYPelsPerMeter=0;
	pbi->biClrUsed=cl;
	pbi->biClrImportant=cl;
}


HBITMAP BMPLoad( const char *filename, SIZE *sz ){
	BITMAPINFOHEADER *pbm;
	HANDLE hFile;
	HDC dc;
	HDC hMemDC;
	HBITMAP hBmp;
	/*  int cx, cy; 2000-06-23 */
	int nPalSize;
	void *pBmp;

	hFile = CreateFile( filename, GENERIC_READ, FILE_SHARE_READ, NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	/*  t@Crbg}bvt@CƉ肵ēǂݍ  */
	pbm = (void*)LoadBMP( hFile );
	CloseHandle( hFile );
	if(!pbm){
		return NULL;
	}

	/*  pbgݒ  */
	dc = GetDC( NULL );
	hMemDC = CreateCompatibleDC( dc );
#if 0
	`悷ƂɃpbgKvAƂ肠B
	if(pbm->biBitCount < 16){
		hPal=CreateDibPalette((BITMAPINFOHEADER *)pbm);
		if( hPal ){
			hpalT=SelectPalette(hMemDC, hPal, FALSE);
			RealizePalette(hMemDC);     // GDI ̃oO?
		}
	}
#endif

	if(16 <= pbm->biBitCount && pbm->biCompression == BI_BITFIELDS){
		nPalSize = 3;
	}else if( pbm->biClrUsed ){
		nPalSize = pbm->biClrUsed;
	}else{
		nPalSize = UsedColor( pbm->biBitCount );
	}
	pBmp=(LPSTR) pbm + pbm->biSize + nPalSize * sizeof(RGBQUAD);

	hBmp = CreateDIBitmap(dc,
		(LPBITMAPINFOHEADER)pbm,
		(LONG)CBM_INIT,
		pBmp,
		(LPBITMAPINFO)pbm,
		DIB_RGB_COLORS);
	ReleaseDC(NULL, dc);
	DeleteDC(hMemDC);

	sz->cx = pbm->biWidth;
	sz->cy = pbm->biHeight;

	return hBmp;
}

static BITMAPINFO *LoadBMP(HANDLE hFile){
	BITMAPFILEHEADER bf;
	BITMAPINFOHEADER bi;
	BITMAPINFO *pbi;
	void *mem;
	int core;
	WORD wPalSize;
	DWORD dwRead;

	/*  t@Cwb_ǂݍ  */
	if(ReadFile(hFile, &bf, sizeof(bf), &dwRead, NULL)){
		if(bf.bfType != BM_SIG){
			/*  t@Cwb_݂  */
			SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
		}
	}

	/*  wb_ǂݍ  */
	if(!ReadFile(hFile, &bi, sizeof(bi), &dwRead, NULL)){
		SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
		return NULL;
	}

	switch(core=bi.biSize){
	case sizeof(BITMAPINFOHEADER):
		/*  gpF  */
		if(bi.biClrUsed==0){
			bi.biClrUsed=UsedColor(bi.biBitCount);
		}
		break;
	case sizeof(BITMAPCOREHEADER):
		/*  ǂݍ񂾂̂COREȂ̂ŁAINFOɕϊ  */
		/*  ꂪRAB  */
		CoreToInfo(&bi);
		/*  INFO߂āACOREi  */
		SetFilePointer(hFile, sizeof(BITMAPCOREHEADER)-sizeof(BITMAPINFOHEADER), NULL, FILE_CURRENT);
		break;
	default:
		/*  Ȃ񂾂  */
		SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
		return NULL;
	}
	/*  l̏ݒ  */
	if( bi.biSizeImage == 0 ){
		bi.biSizeImage = WIDTHBYTES( (DWORD)bi.biWidth * bi.biBitCount ) * bi.biHeight;
	}

	/*  J[e[uǂݍ߂xɊm  */
	if(16<=bi.biBitCount && bi.biCompression == BI_BITFIELDS && bi.biClrUsed==0){
		wPalSize = 3 * sizeof(DWORD);
	}else{
		wPalSize=(WORD)(bi.biClrUsed*sizeof(RGBQUAD));
	}

	if(wPalSize){
		/*  J[e[u  */
		RGBQUAD *quad;
		RGBTRIPLE *trip;
		int i;

		pbi = HeapAlloc( GetProcessHeap(), 0, sizeof(BITMAPINFOHEADER) + wPalSize );
		if(!pbi){
			/*  mۂłȂ  */
			SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
			return NULL;
		}
		/*  J[e[uǂݍ  */
		if(core==sizeof(BITMAPCOREHEADER)){
			/*  RA  */
			i=bi.biClrUsed;
			trip=(RGBTRIPLE *)((LPSTR)pbi + bi.biSize);
			ReadFile(hFile, trip, i*sizeof(RGBTRIPLE), &dwRead, NULL);
			quad=((RGBQUAD *)trip)+i;
			trip+=i;
			while(0<i--){
				quad->rgbRed=trip->rgbtRed;
				quad->rgbBlue=trip->rgbtBlue;
				quad->rgbGreen=trip->rgbtGreen;
				quad->rgbReserved = (BYTE)0;
				quad--;
				trip--;
			}
		}else{
			quad=(RGBQUAD *)((LPSTR)pbi + bi.biSize);
			ReadFile(hFile, quad, wPalSize, &dwRead, NULL);
		}
	}else{
		pbi=NULL;
		wPalSize=0;
	}

	/*  rbg}bvp^[̂߂ɗ̈g  */
	if(pbi){
		mem = HeapReAlloc( GetProcessHeap(), 0, pbi, bi.biSize+bi.biSizeImage+wPalSize );
		if(mem){
			pbi=mem;
		}else{
			HeapFree( GetProcessHeap(), 0, pbi);
			pbi=NULL;
		}
	}else{
		pbi = HeapAlloc( GetProcessHeap(), 0, (bi.biSize+bi.biSizeImage+wPalSize) );
	}
	if(!pbi){
		SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
		return NULL;
	}
	/*  rbg}bvp^[ǂݍ  */
	ReadFile(hFile, (char *)pbi+bi.biSize+wPalSize, bi.biSizeImage, &dwRead, NULL);
	*((BITMAPINFOHEADER*)pbi)=bi;

	/*    */
	return pbi;
}
