/*  ClockViewer for Win32  */
/*  copyright (c) 2005 Bee! (mailto:bee@mbj.nifty.com)  */

#include "base.h"
#include <mmsystem.h>
#include <pbt.h>


static RECT vRC;
HWND vhwndClock;

static HDC vhdcBack;
static HDC vhdcFont;

static HBITMAP vhbmpBack;
static HBITMAP vhbmpFontBack;

static void OnPaint( HWND hwnd, HDC dc );
static void OnCommand( HWND hwnd, WPARAM wParam, LPARAM lParam );
static void OnDestroy( HWND hwnd );
static void OnKeyDown(HWND hwnd, WPARAM wParam, LPARAM lParam );
void OnPower( HWND hwnd, DWORD dwEvent, DWORD dwData );

void ClockWindow( HWND hwndParent ){
	HDC dc;
	HWND hwnd;
	SIZE sz;
	HBITMAP back;
	HBITMAP font;

	if( vhwndClock ){
		DestroyWindow( vhwndClock );
		return;
	}

	font = BMPLoad( "./number.bmp", &sz );
	if( !font ){
		return ;
	}
	vnCharSizeX = sz.cx / 12;
	vnCharSizeY = sz.cy;

	StartInspeceClock();

	hwnd = CreateWindowEx(
		WS_EX_TOOLWINDOW | WS_EX_TOPMOST,
		WINDOW_CLASS, "BClock", WS_POPUP,
		CW_USEDEFAULT, 0, vnCharSizeX * 8, sz.cy, hwndParent, NULL, GetModuleHandle( NULL ), NULL);

	if(!hwnd){
		return;
	}
	SetWindowLong( hwnd, GWL_WNDPROC, (long int)ClockWndProc );

	dc = GetDC( hwnd );
	vhdcBack = CreateCompatibleDC( dc );
	vhdcFont = CreateCompatibleDC( dc );
	ReleaseDC( dc, hwnd );

	back = CreateCompatibleBitmap( dc, vnCharSizeX*8, vnCharSizeY );
	
	vhbmpBack = SelectObject( vhdcBack, back );
	vhbmpFontBack = SelectObject( vhdcFont, font );

	ShowWindow( hwnd, SW_SHOW );
	UpdateWindow( hwnd );
	vhwndClock = hwnd;

}

void UpdateClock( void ){
	static const char zero[] = "0000";
	HDC dc;
	char buf[8];
	int n;
	DWORD dwTime;
	ULARGE_INTEGER tmp;
	int i;
	const char *p;

	if( vhwndClock == NULL ){
		return;
	}

	EnterCriticalSection( &vLaps.lock );

	dwTime = vLaps.item[ vLaps.cur ].tm - vLaps.item[ vLaps.start ].tm;
	tmp = vLaps.item[ vLaps.cur ].clk;
	llsub( &tmp, &vLaps.item[ vLaps.start ].clk );

	LeaveCriticalSection( &vLaps.lock );

	dc = GetDC( vhwndClock );
	FillRect( vhwndClock, &vRC, GetStockObject( WHITE_BRUSH ) );
	n = 0;
	if( dwTime ){
		n = lldiv( &tmp, dwTime );
	}

	if( n ){
		n = ll2str( n, 0, buf, 8 );
		p = buf + n;
		n--;
	}else{
		p = zero;
		n = 4;
	}
	for( i = 0; i < 8; i++ ){
		if( i < n ){
			BitBlt( vhdcBack, vnCharSizeX * i, 0, vnCharSizeX, vnCharSizeY, vhdcFont, vnCharSizeX * 11, 0, SRCCOPY );
		}else{
			if( i == 4 ){
				BitBlt( vhdcBack, vnCharSizeX * i, 0, vnCharSizeX, vnCharSizeY, vhdcFont, vnCharSizeX * 10, 0, SRCCOPY );
			}else{
				BitBlt( vhdcBack, vnCharSizeX * i, 0, vnCharSizeX, vnCharSizeY, vhdcFont, vnCharSizeX *  *p, 0, SRCCOPY );
				p++;
			}
		}
	}

	BitBlt( dc, 0, 0, 8*vnCharSizeX, vnCharSizeY, vhdcBack, 0, 0, SRCCOPY );

	ReleaseDC( vhwndClock, dc );

}

LRESULT CALLBACK ClockWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam){
	PAINTSTRUCT ps;
	HDC dc;
	int n;
	POINT pt;

	switch(uMsg){
	case WM_PAINT:
		dc = BeginPaint(hwnd, &ps);
		OnPaint(hwnd, dc);
		EndPaint(hwnd, &ps);
		break;

	case WM_SIZE:
		GetClientRect( hwnd, &vRC );
		break;

	case WM_DESTROY:
		OnDestroy( hwnd );
		break;

	case WM_KEYDOWN:
		OnKeyDown( hwnd, wParam, lParam );
		break;

	case WM_NCRBUTTONDOWN:
		GetCursorPos( &pt );
		DoAdjustCPU( hwnd, &pt );
		break;

	case WM_NCHITTEST:
		n = DefWindowProc(hwnd, uMsg, wParam, lParam);
		if( n == HTCLIENT ){
			return HTCAPTION;
		}
		return n;

	case WM_COMMAND:
		OnCommand( hwnd, wParam, lParam );
		break;

	case WM_POWERBROADCAST:
		OnPower( hwnd, wParam, lParam );
		break;

	default:
		return DefWindowProc(hwnd, uMsg, wParam, lParam);
	}
	return 0;
}


void OnDestroy( HWND hwnd ){
	HBITMAP back;
	HBITMAP font;

	vhwndClock = NULL;
	StopInspeceClock();
	back = SelectObject( vhdcBack, vhbmpBack );
	font = SelectObject( vhdcFont, vhbmpFontBack );
	DeleteObject( back );
	DeleteObject( font );
	DeleteDC( vhdcBack );
	DeleteDC( vhdcFont );
}


void OnPaint(HWND hwnd, HDC dc){
	BitBlt( dc, 0, 0, 8*vnCharSizeX, vnCharSizeY, vhdcBack, 0, 0, SRCCOPY );
	return;
}


void OnKeyDown(HWND hwnd, WPARAM wParam, LPARAM lParam ){
	POINT pt;

	if( (lParam >> 30) & 0x01 ){
		return;
	}

	pt.x = 0;
	pt.y = 0;
	ClientToScreen( hwnd, &pt );
	DoAdjustCPU( hwnd, &pt );
}

static void OnCommand( HWND hwnd, WPARAM wParam, LPARAM lParam ){
	switch( wParam ){
	case IDM_CPUSTATE:
		SetCpuState( lParam );
		break;
	}
}


void OnPower( HWND hwnd, DWORD dwEvent, DWORD dwData ){
	switch( dwEvent ){
	case PBT_APMSUSPEND:
		StopInspeceClock();
		break;

	case PBT_APMRESUMESUSPEND:
		StartInspeceClock();
		break;
	}
}

